#!/bin/sh
# -*- tab-width: 4 -*- ;; Emacs
# vi: set noexpandtab  :: Vi/ViM
############################################################ INFORMATION
#
# Command Usage:
#
#   ./create.sh [OPTIONS] package-dir [package-dir ...]
#
#   OPTIONS:
#   	-h     Print this message to stderr and exit.
#   	-f     Force. Allow creation over a directory that already exists.
#
############################################################ GLOBALS

#
# Global exit status variables
#
SUCCESS=0
FAILURE=1

#
# Program name and directory
#
progname="${0##*/}"
progdir="${0%/*}"

#
# Options
#
FORCE=

############################################################ FUNCTIONS

# fprintf $fd $fmt [ $opts ... ]
#
# Like printf, except allows you to print to a specific file-descriptor. Useful
# for printing to stderr (fd=2) or some other known file-descriptor.
#
fprintf()
{
	local fd=$1
	[ $# -gt 1 ] || return $FAILURE
	shift 1
	printf "$@" >&$fd
}

# eprintf $fmt [ $opts ... ]
#
# Print a message to stderr (fd=2).
#
eprintf()
{
	fprintf 2 "$@"
}

# die [ $fmt [ $opts ... ]]
#
# Optionally print a message to stderr before exiting with failure status.
#
die()
{
	local fmt="$1"
	[ $# -gt 0 ] && shift 1
	[  "$fmt"  ] && eprintf "$fmt\n" "$@"

	exit $FAILURE
}

# usage
#
# Prints a short syntax statement and exits.
#
usage()
{
	local optfmt="\t%-7s%s\n"

	eprintf "Usage: %s [OPTIONS] package-dir [package-dir ...]\n" \
	        "$progname"

	eprintf "OPTIONS:\n"
	eprintf "$optfmt" "-h" \
	        "Print this message to stderr and exit."
	eprintf "$optfmt" "-f" \
	        "Force. Allow creation over a directory that already exists."

	die
}

############################################################ MAIN

#
# Perform sanity checks
#
[ $# -gt 0 ] || usage

#
# Process command-line options
#
while getopts hoaf flag; do
	case "$flag" in
	h|\?) usage;;
	f) FORCE=1;;
	esac
done
shift $(( $OPTIND - 1 ))

#
# Perform [another] sanity check
#
[ $# -gt 0 ] || usage

#
# Loop over each remaining package-dir argument(s)
#
while [ $# -gt 0 ]; do

	DEST="$1"
	shift 1

	#
	# Get the package name and proper Makefile from pathname
	#
	NAME="${DEST##*/}"
	if [ "$DEST" -a ! "$NAME" ]; then
		DEST="${DEST%/*}"
		NAME="${DEST##*/}"
	fi
	if [ "$NAME" ]; then
		printf "===> Creating \`%s'\n" "$DEST"
	else
		usage
	fi

	#
	# Detect older package creation (used later in Makefile fixup)
	#
	ar_opt=J
	case "$( readlink -f "$DEST" )" in
	*/RELENG_[1-4][_/]*) ar_opt=z ;;
	*/RELENG_[5-9][_/]*) ar_opt=j ;;
	esac

	#
	# Make sure that the directory we are going to create doesn't already
	# exist. If it does, issue an error and skip this package (unless `-f'
	# is passed).
	# 
	# Otherwise, create the destination.
	#
	printf "Creating package repository directory: "
	if [ -e "$DEST" ]; then
		printf "\n"
		eprintf "ERROR: Directory \`%s' already exists\n" "$DEST"
		if [ ! "$FORCE" ]; then
			eprintf "ERROR: Skipping package %s\n" \
			        "(use \`-f' to override)"
			continue
		else
			eprintf "ERROR: Proceeding anyway (\`-f' was passed)\n"
		fi
	fi
	if ! mkdir -p "$DEST"; then
		printf "\n"
		eprintf "ERROR: Could not create directory \`%s'\n" "$DEST"
		die "ERROR: Exiting"
	fi
	printf "%s\n" "$DEST"

	#
	# Create the `src' directory within the package repository
	#
	printf "Creating package \`src' directory...\n"
	if ! mkdir -p "$DEST/src"; then
		eprintf "ERROR: Could not create directory \`%s/src'\n" "$DEST"
		die "ERROR: Exiting"
	fi

	#
	# Extract skeleton directory into package repository
	#
	printf "Copying \`skel' structure into package repository...\n"
	tar co --exclude CVS -f - -C "$progdir/skel" . | tar xkvf - -C "$DEST"

	#
	# Makefile fixup: move in the appropriate Makefile
	#
	printf "Adjusting for archive format...\n"
	case "$ar_opt" in
	z) if [ ! -e "$DEST/Makefile" ] || \
	      cmp "$DEST/Makefile" "$progdir/skel/Makefile"; then
	   	mv -vf "$DEST/Makefile.old" "$DEST/Makefile"
	   else
	   	rm -vf "$DEST/Makefile.old"
	   fi
	   rm -vf "$DEST/Makefile.ng" "$DEST/MANIFEST"
	   ;;
	j) rm -vf "$DEST/Makefile.old" "$DEST/Makefile.ng" "$DEST/MANIFEST" ;;
	J) rm -vf "$DEST/Makefile.old"
	   if [ ! -e "$DEST/Makefile" ] || \
		  cmp "$DEST/Makefile" "$progdir/skel/Makefile"; then
	   	mv -vf "$DEST/Makefile.ng" "$DEST/Makefile"
	   else
	   	rm -vf "$DEST/Makefile.ng"
	   fi
	   rm -vf "$DEST/PLIST" "$DEST/src/+COMMENT" "$DEST/src/+DESC"
	esac

	#
	# That's it (onto the next, back at the top).
	#
	printf "Done.\n"
done

################################################################################
# END
################################################################################
#
# $Header$
#
# $Copyright: 1999-2016 Devin Teske. All rights reserved. $
#
################################################################################
