#!/bin/sh
# -*- tab-width: 4 -*- ;; Emacs
# vi: set noexpandtab  :: Vi/ViM
############################################################ INFORMATION
#
# Command Usage:
#
#   ./create.sh [OPTIONS] package-dir [package-dir ...]
#
#   OPTIONS:
#   	-h     Print this message to stderr and exit.
#   	-f     Force. Allow creation over a directory that already exists.
#
############################################################ CONFIGURATION

#
# Default `License' value for new SPECFILE
#
LICENSE="BSD"

#
# Default `Version' value for new SPECFILE
#
VERSION="1.0"

#
# Default `Release' value for new SPECFILE
#
RELEASE="1"

#
# Default `Buildarch' value for new SPECFILE
#
ARCH="noarch"

#
# Default `URL' value for new SPECFILE
#
URL="http://druidbsd.sf.net/"

#
# Default `Packager' value for new SPECFILE
#
PACKAGER="First Last <first.last@fisglobal.com>"

#
# Default `Vendor' value for new SPECFILE
#
VENDOR="DruidBSD, http://druidbsd.sf.net/"

#
# Default `Changelog' information for new SPECFILE
#
CHANGELOGTIME_day=$( date "+%a %b %m %Y" )
CHANGELOGNAME="First Last <first.last@fisglobal.com> 1.0-1"
CHANGELOGTEXT="- Package created."

############################################################ GLOBALS

#
# Global exit status variables
#
SUCCESS=0
FAILURE=1

#
# Program name and directory
#
progname="${0##*/}"
progdir="${0%/*}"

#
# Options
#
FORCE=

############################################################ FUNCTIONS

# fprintf $fd $fmt [ $opts ... ]
#
# Like printf, except allows you to print to a specific file-descriptor. Useful
# for printing to stderr (fd=2) or some other known file-descriptor.
#
fprintf()
{
	local fd=$1
	[ $# -gt 1 ] || return $FAILURE
	shift 1
	printf "$@" >&$fd
}

# eprintf $fmt [ $opts ... ]
#
# Print a message to stderr (fd=2).
#
eprintf()
{
	fprintf 2 "$@"
}

# die [ $fmt [ $opts ... ]]
#
# Optionally print a message to stderr before exiting with failure status.
#
die()
{
	local fmt="$1"
	[ $# -gt 0 ] && shift 1
	[  "$fmt"  ] && eprintf "$fmt\n" "$@"

	exit $FAILURE
}

# usage
#
# Prints a short syntax statement and exits.
#
usage()
{
	local optfmt="\t%-7s%s\n"

	eprintf "Usage: %s [OPTIONS] package-dir [package-dir ...]\n" \
	        "$progname"

	eprintf "OPTIONS:\n"
	eprintf "$optfmt" "-h" \
	        "Print this message to stderr and exit."
	eprintf "$optfmt" "-f" \
	        "Force. Allow creation over a directory that already exists."

	die
}

############################################################ MAIN

#
# Perform sanity checks
#
[ $# -gt 0 ] || usage

#
# Process command-line options
#
while getopts hgaf flag; do
	case "$flag" in
	h|\?) usage;;
	f) FORCE=1;;
	esac
done
shift $(( $OPTIND - 1 ))

#
# Perform [another] sanity check
#
[ $# -gt 0 ] || usage

#
# Build a list of querytags found in the template.spec file.
#
querytags=$( awk -v regex="@[[:alnum:]]+(:[[:alnum:]]+)?@" '
{
	while (match($0, regex))
	{
		tag = substr($0, RSTART + 1, RLENGTH - 2)
		print tag
		sub(regex, "")
	}
}' < "$progdir/Mk/template.spec" )

#
# Initialize querytag values for generating new/clean template.spec file(s).
#
for tag in $querytags; do
	case "$tag" in
	*:*) varname="${tag%:*}_${tag#*:}";;
	*) varname="$tag";;
	esac
	case "$varname" in
	LICENSE|VERSION|RELEASE|ARCH|URL|PACKAGER|VENDOR) 
		: see configuration section for default values
		export $varname;;
	CHANGELOGTIME_day|CHANGELOGNAME|CHANGELOGTEXT)
		: see configuration section for default values
		export $varname;;
	*)
		export $varname=
	esac
done

#
# Loop over each remaining package-dir argument(s)
#
while [ $# -gt 0 ]; do

	DEST="$1"
	shift 1
	
	#
	# Get the package name and group from pathname
	#
	NAME="${DEST##*/}"
	if [ "$DEST" -a ! "$NAME" ]; then
		DEST="${DEST%/*}"
		NAME="${DEST##*/}"
	fi
	if [ "$NAME" ]; then
		printf "===> Creating \`%s'\n" "$DEST"
	else
		usage
	fi
	case "$DEST" in
	*/*)
		GROUP=$( awk -v path="${DEST%/*}" '
			BEGIN { plen = length(path) }
			{
				line = $0
				gsub(/[[:space:]]+/, "_", line)
				llen = length(line)
				if ( plen < llen ) next

				text2match = ( plen == llen ) ? ( "/" path ) :
					substr(path, (plen + 1) - (llen + 1))

				if ( "/" line == text2match )
				{
					print $0
					exit
				}
			}
		' "$progdir/Mk/GROUPS" )
		;;
	*)
		GROUP=
	esac
	printf "Package NAME: %s\n" "$NAME"
	printf "Package GROUP: %s\n" "$GROUP"

	#
	# Make sure that the directory we are going to create doesn't already
	# exist. If it does, issue an error and skip this package (unless `-f'
	# is passed).
	# 
	# Otherwise, create the destination.
	#
	printf "Creating package repository directory: "
	if [ -e "$DEST" ]; then
		printf "\n"
		eprintf "ERROR: Directory \`%s' already exists\n" "$DEST"
		if [ ! "$FORCE" ]; then
			eprintf "ERROR: Skipping package %s\n" \
			        "(use \`-f' to override)"
			continue
		else
			eprintf "ERROR: Proceeding anyway (\`-f' was passed)\n"
		fi
	fi
	if ! mkdir -p "$DEST"; then
		printf "\n"
		eprintf "ERROR: Could not create directory \`%s'\n" "$DEST"
		die "ERROR: Exiting"
	fi
	printf "%s\n" "$DEST"

	#
	# Create the `src' directory within the package repository
	#
	printf "Creating package \`src' directory...\n"
	if ! mkdir -p "$DEST/src"; then
		eprintf "ERROR: Could not create directory \`%s/src'\n" "$DEST"
		die "ERROR: Exiting"
	fi

	#
	# Extract skeleton directory into package repository
	#
	printf "Copying \`skel' structure into package repository...\n"
	tar co --exclude CVS -f - -C "$progdir/skel" . | tar xkvf - -C "$DEST"

	#
	# Declare initial dependencies and file-list to be null
	#
	export REQUIRES= FILE_LISTING=

	#
	# Declare initial summary/description to be the same as the name
	#
	export NAME GROUP
	SUMMARY="$NAME"
	DESCRIPTION="$NAME"
	export SUMMARY DESCRIPTION

	#
	# Generate the spec-file
	#
	printf "Generating \`%s' from \`%s'...\n" \
	       "$DEST/SPECFILE" "$progdir/Mk/template.spec"
	if ! awk -vregex="[[:alnum:]]+(:[[:alnum:]]+)?" '
	{
		if ( $0 ~ /^__REQUIRES__$/ )
		{
			printf "%s\n", ENVIRON["REQUIRES"]
			next
		}
		if ( $0 ~ /^__FILE_LISTING__$/ )
		{
			printf "%s\n", ENVIRON["FILE_LISTING"]
			next
		}

		n = split($0, line_buffer, /@/)
		if (n >= 3)
		{
			s = 1
			printf "%s", line_buffer[s]
			while (s++ < n)
			{
				if (line_buffer[s] ~ regex)
				{
					varname = line_buffer[s]
					sub(/:/, "_", varname)
					printf "%s", ENVIRON[varname]
					s++
					printf "%s", line_buffer[s]
				}
				else printf "%s", line_buffer[s]
			}
			printf "\n"
		}
		else print
	}
	' < "$progdir/Mk/template.spec" > "$DEST/SPECFILE"; then
		eprintf "ERROR: Could not create \`$DEST/SPECFILE'"
		die "ERROR: Exiting"
	fi

	#
	# That's it (onto the next, back at the top).
	#
	printf "Done.\n"
done

################################################################################
# END
################################################################################
#
# $Header$
#
# $Copyright: 1999-2016 Devin Teske. All rights reserved. $
#
################################################################################
