#!/bin/sh
# -*- tab-width: 4 -*- ;; Emacs
# vi: set noexpandtab  :: Vi/ViM
############################################################ INFORMATION
#
# Command Usage:
#
#   ./unpack.sh [OPTIONS] package-archive [package-archive ...]
#
#   OPTIONS:
#   	-h     Print this message to stderr and exit.
#   	-g     Unpack archive to group sub-directory. Default is to unpack
#   	       to the local directory.
#   	-a     Move the package-archive to an `archive' sub-directory (next
#   	       to the unpacked `src' directory) after successfully unpacking
#   	       the package. Not performed by default.
#   	-f     Force. Allow unpacking to a directory that already exists.
#
############################################################ GLOBALS

#
# Global exit status variables
#
SUCCESS=0
FAILURE=1

#
# Program name and directory
#
progname="${0##*/}"
progdir="${0%/*}"

#
# OS Glue
#
UNAME_S=$( uname -s )

#
# Options
#
MOVE_PACKAGE=
USE_GROUP_SUBDIR=
FORCE=

############################################################ FUNCTIONS

# fprintf $fd $fmt [ $opts ... ]
#
# Like printf, except allows you to print to a specific file-descriptor. Useful
# for printing to stderr (fd=2) or some other known file-descriptor.
#
fprintf()
{
	local fd=$1
	[ $# -gt 1 ] || return $FAILURE
	shift 1
	printf "$@" >&$fd
}

# eprintf $fmt [ $opts ... ]
#
# Print a message to stderr (fd=2).
#
eprintf()
{
	fprintf 2 "$@"
}

# die [ $fmt [ $opts ... ]]
#
# Optionally print a message to stderr before exiting with failure status.
#
die()
{
	local fmt="$1"
	[ $# -gt 0 ] && shift 1
	[  "$fmt"  ] && eprintf "$fmt\n" "$@"

	exit $FAILURE
}

# usage
#
# Prints a short syntax statement and exits.
#
usage()
{
	local optfmt="\t%-7s%s\n"

	eprintf "Usage: %s [OPTIONS] package-archive [package-archive ...]\n" \
	        "$progname"

	eprintf "OPTIONS:\n"
	eprintf "$optfmt" "-h" \
	        "Print this message to stderr and exit."
	eprintf "$optfmt" "-g" \
	        "Unpack archive to group sub-directory. Default is to unpack"
	eprintf "$optfmt" "" \
	        "to the local directory."
	eprintf "$optfmt" "-a" \
	        "Move the package-archive to an \`archive' sub-directory (next"
	eprintf "$optfmt" "" \
	        "to the unpacked \`src' directory) after successfully unpacking"
	eprintf "$optfmt" "" \
	        "the package. Not performed by default."
	eprintf "$optfmt" "-f" \
	        "Force. Allow unpacking to a directory that already exists."

	die
}

# rpm_get_group $rpm_archive_file
#
# Gets the group from an RPM archive.
#
rpm_get_group()
{
	local file="$1"

	#
	# Sanity check
	#
	[ $# -gt 0 -a "$file" ] || return $SUCCESS

	rpm -q -p "$file" --qf "%{GROUP}\n" | sed -e 's/[[:space:]]/_/g'
}

# rpm_get_name $rpm_archive_file
#
# Gets the name from an RPM archive.
#
rpm_get_name()
{
	local file="$1"

	#
	# Sanity check
	#
	[ $# -gt 0 -a "$file" ] || return $SUCCESS

	rpm -q -p "$file" --qf "%{NAME}\n"
}

############################################################ MAIN

#
# Perform sanity checks
#
[ $# -gt 0 ] || usage

#
# Process command-line options
#
while getopts hgaf flag; do
	case "$flag" in
	h|\?) usage;;
	g) USE_GROUP_SUBDIR=1;;
	a) MOVE_PACKAGE=1;;
	f) FORCE=1;;
	esac
done
shift $(( $OPTIND - 1 ))

#
# Perform [another] sanity check
#
[ $# -gt 0 ] || usage

#
# Build a list of querytags found in the template.spec file.
#
querytags=$( awk -v regex="@[[:alnum:]]+(:[[:alnum:]]+)?@" '
{
	while (match($0, regex))
	{
		tag = substr($0, RSTART + 1, RLENGTH - 2)
		print tag
		sub(regex, "")
	}
}' < $progdir/Mk/template.spec )

#
# Loop over each remaining package-archive argument(s)
#
while [ $# -gt 0 ]; do

	package="$1"
	shift 1
	
	printf "===> Unpacking \`%s'\n" "$package"

	#
	# Get the package group and name
	#
	printf "Reading NAME from package: "
	NAME=$(rpm_get_name "$package") || die
	if [ ! "$NAME" ]; then
		eprintf "ERROR: No NAME recorded for package %s\n" \
		        "(skipping package)"
		continue
	else
		printf "%s\n" "$NAME"
	fi
	printf "Reading GROUP from package: "
	GROUP=$(rpm_get_group "$package")
	if [ ! "$GROUP" ]; then
		printf "\n"
		eprintf "ERROR: No GROUP recorded for package\n"
		eprintf "ERROR: Unable to unpack \`%s'\n" "$package"

		read -p "Please provide a path to unpack to: " GROUP
		if [ ! "$GROUP" ]; then
			eprintf "ERROR: No path provided (skipping package)\n"
			continue
		fi
		USE_GROUP_SUBDIR=1
	else
		printf "%s\n" "$GROUP"
	fi

	#
	# Determine where we will unpack the package to.
	#
	if [ "$USE_GROUP_SUBDIR" ]; then
		DEST="$GROUP/$NAME"
	else
		DEST="$NAME"
	fi

	#
	# Make sure that the directory in which we are going to unpack to
	# doesn't already exist. If it does, issue an error and skip this
	# package (unless `-f' is passed).
	# 
	# Otherwise, create the destination and proceed with unpacking.
	#
	printf "Creating package repository directory: "
	if [ -e "$DEST" ]; then
		printf "\n"
		eprintf "ERROR: Directory \`%s' already exists\n" "$DEST"
		if [ ! "$FORCE" ]; then
			eprintf "ERROR: Skipping package %s\n" \
			        "(use \`-f' to override)"
			continue
		else
			eprintf "ERROR: Proceeding anyway (\`-f' was passed)\n"
		fi
	fi
	if ! mkdir -p "$DEST"; then
		printf "\n"
		eprintf "ERROR: Could not create directory \`%s'\n" "$DEST"
		die "ERROR: Exiting"
	fi
	printf "%s\n" "$DEST"

	#
	# Unpack the package
	#
	printf "Unpacking...\n"
	if ! mkdir -p "$DEST/src"; then
		eprintf "ERROR: Could not create directory \`%s/src'\n" "$DEST"
		die "ERROR: Exiting"
	fi
	no_absolute_filenames=
	[ "$UNAME_S" = "Linux" ] \
		&& no_absolute_filenames=--no-absolute-filenames
	if ! rpm2cpio "$package" | ( cd "$DEST/src" && cpio -idmu \
	   $no_absolute_filenames --no-preserve-owner ); then
		eprintf "ERROR: Could not unpack \`%s' to \`%s/src'" \
		        "$package" "$DEST"
		die "ERROR: Exiting"
	fi

	#
	# Obtain querytag values from current package
	#
	for tag in $querytags; do
		case "$tag" in
		*:*) varname="${tag%:*}_${tag#*:}";;
		*) varname="$tag";;
		esac
		eval $varname='$(rpm -q -p "$package" --qf "%{'$tag'}")'
		export $varname

		#
		# Nullify unused fields with a value of "(none)"
		#
		value=$( eval echo "\"\$$varname\"" )
		[ "$value" = "(none)" ] && eval $varname=
	done

	#
	# Generate the spec-file
	#
	printf "Generating \`%s' from \`%s'...\n" \
	       "$DEST/SPECFILE" "$progdir/Mk/template.spec"
	REQUIRES=$( rpm -qRp "$package" |
	            	awk '!/^(\/|[[:alpha:]]+\()/{print "Requires:", $0}' )
	FILE_LISTING=$( find "$DEST/src" ! -type d |
	                	sed -e "s#^$DEST/src##" )
	export REQUIRES FILE_LISTING
	if ! awk -vregex="[[:alnum:]]+(:[[:alnum:]]+)?" '
	{
		if ( $0 ~ /^__REQUIRES__$/ )
		{
			printf "%s\n", ENVIRON["REQUIRES"]
			next
		}
		if ( $0 ~ /^__FILE_LISTING__$/ )
		{
			printf "%s\n", ENVIRON["FILE_LISTING"]
			next
		}

		n = split($0, line_buffer, /@/)
		if (n >= 3)
		{
			s = 1
			printf "%s", line_buffer[s]
			while (s++ < n)
			{
				if (line_buffer[s] ~ regex)
				{
					varname = line_buffer[s]
					sub(/:/, "_", varname)
					printf "%s", ENVIRON[varname]
					s++
					printf "%s", line_buffer[s]
				}
				else printf "%s", line_buffer[s]
			}
			printf "\n"
		}
		else print
	}
	' < "$progdir/Mk/template.spec" > "$DEST/SPECFILE"; then
		eprintf "ERROR: Could not create \`$DEST/SPECFILE'"
		die "ERROR: Exiting"
	fi

	#
	# Archive package
	#
	if [ "$MOVE_PACKAGE" = "1" ]; then
		echo "Archiving package..."
		if ! mkdir "$DEST/archive"; then
			eprintf "ERROR: Unable to create %s \`%s/archive'" \
			        "directory" "$DEST"
			die "ERROR: Exiting"
		fi
		if ! mv "$package" "$DEST/archive"; then
			eprintf "ERROR: Unable to %s \`%s' to \`%s/archive'" \
			        "move" "$package" "$DEST"
			die "ERROR: Exiting"
		fi
	fi

	#
	# Extract skeleton directory into package repository
	#
	printf "Copying \`skel' structure into package repository...\n"
	tar co --exclude CVS -f - -C "$progdir/skel" . | tar xkvf - -C "$DEST"

	#
	# That's it (onto the next, back at the top).
	#
	printf "Done.\n"
done

################################################################################
# END
################################################################################
#
# $Header$
#
# $Copyright: 1999-2016 Devin Teske. All rights reserved. $
#
################################################################################
